<?php
abstract class db_adapter {
    var $prefix;
    var $query_count = 0;

    /**
     * Set the prefix that should be used.
     * e.g. joomla uses only tables starting with 'jos_', thus 
     * set_prefix('jos_') can be used for simplification.
     */
    function set_prefix($prefix) {
        $this->prefix = $prefix;
    }

    /**
     * Log and count one query
     */
    function log_query($query) {
        //echo $query."\n";
        $this->query_count++;
    }

    /**
     * Get the LIMIT part of an SQL query.
     * @param $limit = null means no limit.
     *   $limit = array('start' => 0, 'count' => 10)
     */
    public function get_limit($limit) {
        // prepare limit
        if ($limit == null) return '';
        assert('isset($limit[\'start\'])');
        $start = $limit['start'];
        assert('is_numeric($start)');
        assert('isset($limit[\'count\'])');
        $count = $limit['count'];
        assert('is_numeric($count)');

        // mysql version
        return " LIMIT $start, $count";
    }

    /**
     * Get the ORDER part of an SQL query
     */
    public function get_order($order) {
        if ($order == null) return '';
        return ' ORDER BY '.$order;
    }

    /**
     * Get the WHERE part of an SQL query
     * @param $order = null means no order.
     *   $order = 'id DESC'
     */
    public function get_where($where) {
        if ($where == null) return '';
        if (is_string($where)) return ' WHERE '.$where;
        if (is_array($where)) {
            if (count($where) == 0) return '';
            $result = '';
            foreach ($where as $k => $v) {
                if (strlen($result) > 0) $result .= ' AND ';
                $result .= '`' . mysql_escape_string($k);
                if (is_string($v)) {
                    $result .= '` = \''.$v.'\'';
                } elseif (is_numeric($v)) {
                    $result .= '` = '.$v;
                } elseif (is_bool($v)) {
                    $result .= ' = '.($v?'TRUE':'FALSE');
                } elseif (is_null($v)) {
                    $result .= '` IS NULL';
                } elseif ($v instanceof database_object) {
                    $result .= '_id` = '.($v->id);
                } else {
                    die('sql WHERE type fail');
                }
            }
            return ' WHERE '.$result;
        }
        die('$where must be null, a string or an array');
    }

    /**
     * Those abstract functions have to be overwritten by sub-classes like 
     * db_adodb and db_mysql.
     */
    abstract function query($query);
    abstract function fetch_columns($table);
    abstract function select_single($query);
    abstract function init();

}

